# Created by Noah Kantrowitz on 2008-05-16.
# Copyright (c) 2008 Noah Kantrowitz. All rights reserved.
import itertools
import re


from trac.core import *
from trac.web.api import ITemplateStreamFilter
from trac.web.chrome import ITemplateProvider, add_stylesheet
from trac.config import Option, OrderedExtensionsOption
from genshi.builder import tag
from genshi.filters.transform import Transformer
from pkg_resources import resource_filename

from hackergotchi.api import IHackergotchiProvider


class HackergotchiModule(Component):
    """A stream filter to add hackergotchi emblems to the timeline."""

    providers = OrderedExtensionsOption('hackergotchi', 'providers',
                                        IHackergotchiProvider,
                                        default='GravatarHackergotchiProvider, IdenticonHackergotchiProvider')

    implements(ITemplateStreamFilter, ITemplateProvider)

    anon_re = re.compile('([^<]+?)\s+<([^>]+)>', re.U)

    # ITemplateStreamFilter methods
    def filter_stream(self, req, method, filename, stream, data):
        if req.path_info.startswith('/timeline'):
            closure_state = [0]
            cache = {}

            def f(stream):
                # Update the closed value
                n = closure_state[0]
                closure_state[0] += 1

                # Extract the user information
                author = data['events'][n]['author'].strip()
                user_info = cache.get(author)
                if user_info is not None:
                    author, name, email = user_info
                else:
                    db = self.env.get_db_cnx()
                    user_info = self._get_info(author, db)
                    cache[author] = user_info
                    author, name, email = user_info

                # Try to find a provider
                for provider in self.providers:
                    href = provider.get_hackergotchi(
                        req.href, author, name, email)
                    if href is not None:
                        break
                else:
                    href = req.href.chrome('hackergotchi', 'default.png')

                # Build our element
                elm = tag.img(src=href, alt='Hackergotchi for %s' % author,
                              class_='hackergotchi')

                # Output the combined stream
                return itertools.chain(elm.generate(), stream)

            stream |= Transformer(
                '//div[@id="content"]/dl/dt/a/span[@class="time"]').filter(f)
            add_stylesheet(req, 'hackergotchi/hackergotchi.css')
        return stream

    # ITemplateProvider methods
    def get_htdocs_dirs(self):
        yield 'hackergotchi', resource_filename(__name__, 'htdocs')

    def get_templates_dirs(self):
        # return [resource_filename(__name__, 'templates')]
        return []

    # Internal methods
    def _get_info(self, author, db):
        if author == 'anonymous':
            # Don't even bother trying for "anonymous"
            return author, None, None

        md = self.anon_re.match(author)
        if md:
            # name <email>
            return 'anonymous', md.group(1), md.group(2)

        cursor = db.cursor()
        cursor.execute('SELECT name, value FROM session_attribute WHERE sid=%s AND authenticated=%s',
                       (author, 1))
        rows = cursor.fetchall()
        if rows:
            # Authenticated user, with session
            name = email = None
            for key, value in rows:
                if key == 'name':
                    name = value
                elif key == 'email':
                    email = value
            if name or email:
                return author, name, email
            else:
                return author, None, None

        # Assume anonymous user from this point on
        if '@' in author:
            # Likely an email address
            return 'anonymous', None, author

        # See if there is a default domain
        domain = self.config.get('notification', 'smtp_default_domain')
        if domain and ' ' not in author:
            return author, None, author + '@' + domain

        return 'anonymous', author, None



# Created by Noah Kantrowitz on 2008-05-16.
# Copyright (c) 2008 Noah Kantrowitz. All rights reserved.
import urllib
from cStringIO import StringIO
try:
    from hashlib import md5
except ImportError:
    from md5 import new as md5

from trac.core import *
from trac.config import Option, BoolOption
from trac.web.api import IRequestHandler, RequestDone

from hackergotchi.api import IHackergotchiProvider
try:
    from hackergotchi.identicon import render_identicon
except ImportError:
    render_identicon = None

# TODO: Add a client-side identicon implementation using canvas <NPK>


class GravatarHackergotchiProvider(Component):
    """Use gravatar.com to provide images."""

    implements(IHackergotchiProvider)

    default = Option('hackergotchi', 'gravatar_default', default='identicon',
                     doc='The default value to pass along to gravatar to use if the email address does not match.')

    # IHackergotchiProvider methods
    def get_hackergotchi(self, href, user, name, email):
        if email:
            return 'http://www.gravatar.com/avatar.php?' + urllib.urlencode({
                'gravatar_id': md5(email).hexdigest(),
                'default': self.default,
                'size': 20,
            })


class IdenticonHackergotchiProvider(Component):
    """Generate identicons locally to provide images."""

    implements(IHackergotchiProvider, IRequestHandler)

    # IHackergotchiProvider methods
    def get_hackergotchi(self, href, user, name, email):
        if render_identicon is None:
            return None

        h = md5(user)
        h.update(name or '')
        h.update(email or '')
        code = h.hexdigest()[:8]
        return href.identicon(code + '.png')

    # IRequestHandler methods
    def match_request(self, req):
        return req.path_info.startswith('/identicon')

    def process_request(self, req):
        if render_identicon is None:
            raise TracError('PIL not installed, can not render identicon')

        # Render the identicon to a string
        code = int(req.path_info[11:-4], 16)
        icon = render_identicon(code, 20)
        out = StringIO()
        icon.save(out, 'png')
        data = out.getvalue()

        # Send response
        req.send_response(200)
        req.send_header('Content-Type', 'image/png')
        req.send_header('Content-Length', len(data))
        req.send_header('Cache-Control', 'max-age=36000')
        if req.method != 'HEAD':
            req.write(data)
        raise RequestDone



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
import os

from setuptools import setup

setup(
    name='TracHackergotchi',
    version='1.2',
    packages=['hackergotchi'],
    package_data={'hackergotchi': ['htdocs/*.js', 'htdocs/*.css']},

    author='Noah Kantrowitz',
    author_email='noah@coderanger.net',
    description='Adds user-specific icons to the Trac timeline view.',
    long_description=open(os.path.join(
        os.path.dirname(__file__), 'README')).read(),
    license='BSD',
    keywords='trac plugin timeline identicon hackergotchi',
    url='http://trac-hacks.org/wiki/HackergotchiPlugin',
    classifiers=[
        'Framework :: Trac',
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],

    install_requires=['Trac'],

    entry_points={
        'trac.plugins': [
            'hackergotchi.api = hackergotchi.api',
            'hackergotchi.providers = hackergotchi.providers',
            'hackergotchi.web_ui = hackergotchi.web_ui',
        ],
    },
)



# Created by Noah Kantrowitz on 2008-05-16.
# Copyright (c) 2008 Noah Kantrowitz. All rights reserved.

from trac.core import *


class IHackergotchiProvider(Interface):
    """An extension-point interface for exposing hackergotchi providers."""

    def get_hackergotchi(href, user, name, email):
        """Return an href to an image corresponding the user information
        given.

        :param href: An Href object for the current Trac.
        :param user: The username or 'anonymous'
        :param name: The user's full name or None
        :param email: The user's email address or None
        """
